@@ -5,6 +5,7 @@ use crate::delimiter::{sniff_delimiter, Delimiter};
 use crate::find;
 use crate::help;
 use crate::input::{Control, InputHandler};
+use crate::sort::{self, SorterStatus};
 use crate::ui::{CsvTable, CsvTableState, FilterColumnsState, FinderState};
 use crate::view;
 
@@ -24,10 +25,10 @@ fn get_offsets_to_make_visible(
     csv_table_state: &CsvTableState,
 ) -> (Option<u64>, Option<u64>) {
     // TODO: row_index() should probably be u64
-    let new_rows_offset = if rows_view.in_view(found_record.row_index() as u64) {
+    let new_rows_offset = if rows_view.in_view(found_record.row_order() as u64) {
         None
     } else {
-        Some(found_record.row_index() as u64)
+        Some(found_record.row_order() as u64)
     };
 
     let cols_offset = csv_table_state.cols_offset;
@@ -50,6 +51,8 @@ fn scroll_to_found_record(
     let (new_rows_offset, new_cols_offset) =
         get_offsets_to_make_visible(&found_record, rows_view, csv_table_state);
 
+    // csv_table_state.debug = format!("{:?} {:?}", new_rows_offset, new_cols_offset);
+    // csv_table_state.debug = format!("{:?}", found_record);
     if let Some(rows_offset) = new_rows_offset {
         rows_view.set_rows_from(rows_offset).unwrap();
         csv_table_state.set_rows_offset(rows_offset);
@@ -104,9 +107,11 @@ pub struct App {
     echo_column: Option<String>,
     ignore_case: bool,
     help_page_state: help::HelpPageState,
+    sorter: Option<Arc<sort::Sorter>>,
 }
 
 impl App {
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         filename: &str,
         delimiter: Delimiter,
@@ -114,6 +119,10 @@ impl App {
         show_stats: bool,
         echo_column: Option<String>,
         ignore_case: bool,
+        no_headers: bool,
+        columns_regex: Option<String>,
+        filter_regex: Option<String>,
+        find_regex: Option<String>,
     ) -> Result<Self> {
         let input_handler = InputHandler::new();
 
@@ -128,7 +137,7 @@ impl App {
             Delimiter::Character(d) => d,
             Delimiter::Auto => sniff_delimiter(filename).unwrap_or(b','),
         };
-        let config = csv::CsvConfig::new(filename, delimiter);
+        let config = csv::CsvConfig::new(filename, delimiter, no_headers);
         let shared_config = Arc::new(config);
 
         let csvlens_reader = csv::CsvLensReader::new(shared_config.clone())
@@ -147,6 +156,7 @@ impl App {
             rows_view.headers().len(),
             &echo_column,
             ignore_case,
+            no_headers,
         );
 
         let finder: Option<find::Finder> = None;
@@ -156,7 +166,7 @@ impl App {
         let transient_message: Option<String> = None;
         let help_page_state = help::HelpPageState::new();
 
-        let app = App {
+        let mut app = App {
             input_handler,
             num_rows_not_visible,
             shared_config,
@@ -170,8 +180,19 @@ impl App {
             echo_column,
             ignore_case,
             help_page_state,
+            sorter: None,
         };
 
+        if let Some(pat) = &columns_regex {
+            app.set_columns_filter(pat);
+        }
+
+        if let Some(pat) = &filter_regex {
+            app.handle_find_or_filter(pat, true);
+        } else if let Some(pat) = &find_regex {
+            app.handle_find_or_filter(pat, false);
+        }
+
         Ok(app)
     }
 
@@ -329,42 +350,10 @@ impl App {
                 }
             }
             Control::Find(s) | Control::Filter(s) => {
-                let re = self.create_regex(s);
-                if let Ok(target) = re {
-                    // TODO: need to reset row views filter if any first?
-                    self.finder =
-                        Some(find::Finder::new(self.shared_config.clone(), target).unwrap());
-                    match control {
-                        Control::Find(_) => {
-                            // will scroll to first result below once ready
-                            self.first_found_scrolled = false;
-                            self.rows_view.reset_filter().unwrap();
-                        }
-                        Control::Filter(_) => {
-                            self.rows_view.set_rows_from(0).unwrap();
-                            self.rows_view
-                                .set_filter(self.finder.as_ref().unwrap())
-                                .unwrap();
-                        }
-                        _ => {}
-                    }
-                } else {
-                    self.finder = None;
-                    // TODO: how to show multi-line error
-                    self.transient_message = Some(format!("Invalid regex: {s}"));
-                }
-                self.csv_table_state.reset_buffer();
+                self.handle_find_or_filter(s, matches!(control, Control::Filter(_)));
             }
-            Control::FilterColumns(s) => {
-                let re = self.create_regex(s);
-                if let Ok(target) = re {
-                    self.rows_view.set_columns_filter(target).unwrap();
-                } else {
-                    self.rows_view.reset_columns_filter().unwrap();
-                    self.transient_message = Some(format!("Invalid regex: {s}"));
-                }
-                self.csv_table_state.reset_buffer();
-                self.csv_table_state.set_cols_offset(0);
+            Control::FilterColumns(pat) => {
+                self.set_columns_filter(pat);
             }
             Control::BufferContent(input) => {
                 self.csv_table_state
@@ -390,6 +379,35 @@ impl App {
                         .replace("Line wrap enabled".to_string());
                 }
             }
+            Control::ToggleSort => {
+                if let Some(selected_column_index) = self.get_global_selected_column_index() {
+                    let mut should_create_new_sorter = false;
+                    if let Some(sorter) = &self.sorter {
+                        if selected_column_index as usize != sorter.column_index {
+                            should_create_new_sorter = true;
+                        }
+                    } else {
+                        should_create_new_sorter = true;
+                    }
+                    if should_create_new_sorter {
+                        let column_name = self
+                            .rows_view
+                            .raw_headers()
+                            .get(selected_column_index as usize)
+                            .cloned()
+                            .unwrap_or_default();
+                        let _sorter = sort::Sorter::new(
+                            self.shared_config.clone(),
+                            selected_column_index as usize,
+                            column_name,
+                        );
+                        self.sorter = Some(Arc::new(_sorter));
+                    }
+                } else {
+                    self.transient_message
+                        .replace("Press TAB and select a column before sorting".to_string());
+                }
+            }
             Control::IncreaseWidth => {
                 self.adjust_column_width(4);
             }
@@ -400,6 +418,7 @@ impl App {
                 self.csv_table_state.column_width_overrides.reset();
                 self.reset_filter();
                 self.rows_view.reset_columns_filter().unwrap();
+                self.reset_sorter();
             }
             Control::UnknownOption(s) => {
                 self.csv_table_state.reset_buffer();
@@ -409,6 +428,42 @@ impl App {
             _ => {}
         }
 
+        if let Some(sorter) = &self.sorter {
+            // Update rows_view sorter if outdated
+            let mut should_set_rows_view_sorter = false;
+            if sorter.status() == SorterStatus::Finished {
+                if let Some(rows_view_sorter) = self.rows_view.sorter() {
+                    if rows_view_sorter.column_index != sorter.column_index {
+                        should_set_rows_view_sorter = true;
+                    }
+                } else {
+                    should_set_rows_view_sorter = true;
+                }
+            }
+            if should_set_rows_view_sorter {
+                self.rows_view.set_sorter(sorter).unwrap();
+            }
+
+            // Update finder if sorter outdated
+            let mut should_create_new_finder = false;
+            if sorter.status() == SorterStatus::Finished {
+                if let Some(finder) = &self.finder {
+                    if let Some(finder_sorter) = finder.sorter() {
+                        if finder_sorter.column_index != sorter.column_index {
+                            should_create_new_finder = true;
+                        }
+                    } else {
+                        should_create_new_finder = true;
+                    }
+                }
+            }
+            if should_create_new_finder {
+                let target = self.finder.as_ref().unwrap().target();
+                let sorter = self.sorter.clone();
+                self.create_finder(target, self.rows_view.is_filter(), sorter);
+            }
+        }
+
         if let Some(fdr) = self.finder.as_mut() {
             if !self.rows_view.is_filter() {
                 // scroll to first result once ready
@@ -426,8 +481,8 @@ impl App {
                 }
 
                 // reset cursor if out of view
-                if let Some(cursor_row_index) = fdr.cursor_row_index() {
-                    if !self.rows_view.in_view(cursor_row_index as u64) {
+                if let Some(cursor_row_order) = fdr.cursor_row_order() {
+                    if !self.rows_view.in_view(cursor_row_order as u64) {
                         fdr.reset_cursor();
                     }
                 }
@@ -474,21 +529,70 @@ impl App {
         self.csv_table_state.filter_columns_state =
             FilterColumnsState::from_rows_view(&self.rows_view);
 
+        self.csv_table_state.update_sorter(&self.sorter);
+
         self.csv_table_state.transient_message = self.transient_message.clone();
 
-        // self.csv_table_state.debug = format!("{:?}", self.csv_table_state.column_width_overrides);
+        // self.csv_table_state.debug = format!("{:?}", self.sorter);
 
         Ok(())
     }
 
-    fn create_regex(&mut self, s: &String) -> std::result::Result<Regex, regex::Error> {
-        let lower_s = s.to_lowercase();
-        let re = if self.ignore_case && lower_s.starts_with(s) {
-            Regex::new(&format!("(?i){}", s.as_str()))
+    fn create_finder(&mut self, target: Regex, is_filter: bool, sorter: Option<Arc<sort::Sorter>>) {
+        let _finder = find::Finder::new(self.shared_config.clone(), target, sorter).unwrap();
+        self.finder = Some(_finder);
+        if is_filter {
+            self.rows_view.set_rows_from(0).unwrap();
+            self.rows_view
+                .set_filter(self.finder.as_ref().unwrap())
+                .unwrap();
         } else {
-            Regex::new(s.as_str())
-        };
-        re
+            // will scroll to first result below once ready
+            self.first_found_scrolled = false;
+            self.rows_view.reset_filter().unwrap();
+        }
+    }
+
+    fn create_regex(&mut self, s: &str) -> std::result::Result<Regex, regex::Error> {
+        let lower_s = s.to_lowercase();
+        if self.ignore_case && lower_s.starts_with(s) {
+            Regex::new(&format!("(?i){}", s))
+        } else {
+            Regex::new(s)
+        }
+    }
+
+    fn set_columns_filter(&mut self, pat: &str) {
+        let re = self.create_regex(pat);
+        if let Ok(target) = re {
+            self.rows_view.set_columns_filter(target).unwrap();
+        } else {
+            self.rows_view.reset_columns_filter().unwrap();
+            self.transient_message = Some(format!("Invalid regex: {pat}"));
+        }
+        self.csv_table_state.reset_buffer();
+        self.csv_table_state.set_cols_offset(0);
+    }
+
+    fn handle_find_or_filter(&mut self, pat: &str, is_filter: bool) {
+        let re = self.create_regex(pat);
+        if let Ok(target) = re {
+            let _sorter = if let Some(s) = &self.sorter {
+                if s.status() == SorterStatus::Finished {
+                    Some(s.clone())
+                } else {
+                    None
+                }
+            } else {
+                None
+            };
+            self.create_finder(target, is_filter, _sorter);
+        } else {
+            self.finder = None;
+            // TODO: how to show multi-line error
+            self.transient_message = Some(format!("Invalid regex: {pat}"));
+        }
+        self.csv_table_state.reset_buffer();
     }
 
     fn increase_cols_offset(&mut self) {
@@ -504,10 +608,7 @@ impl App {
     }
 
     fn adjust_column_width(&mut self, delta: i16) {
-        // local index as in local to the view port
-        if let Some(local_column_index) = self.rows_view.selection.column.index() {
-            let column_index = local_column_index.saturating_add(self.csv_table_state.cols_offset);
-
+        if let Some(column_index) = self.get_selected_column_index() {
             if let Some(view_layout) = &mut self.csv_table_state.view_layout {
                 let current_width = view_layout.column_widths[column_index as usize];
                 let new_width = (current_width as i16).saturating_add(delta);
@@ -524,6 +625,20 @@ impl App {
         }
     }
 
+    fn get_selected_column_index(&self) -> Option<u64> {
+        // local index as in local to the view port
+        if let Some(local_column_index) = self.rows_view.selection.column.index() {
+            return Some(local_column_index.saturating_add(self.csv_table_state.cols_offset));
+        }
+        None
+    }
+
+    fn get_global_selected_column_index(&self) -> Option<u64> {
+        // TODO: maybe this and above should be methods provided by RowsView directly?
+        self.get_selected_column_index()
+            .map(|local_index| self.rows_view.get_column_origin_index(local_index as usize) as u64)
+    }
+
     fn reset_filter(&mut self) {
         if self.finder.is_some() {
             self.finder = None;
@@ -532,6 +647,12 @@ impl App {
         }
     }
 
+    fn reset_sorter(&mut self) {
+        // TODO: consolidate rows_view reset
+        self.sorter = None;
+        self.rows_view.reset_sorter().unwrap();
+    }
+
     fn render_frame(&mut self, f: &mut Frame) {
         let size = f.size();
 
@@ -576,6 +697,76 @@ mod tests {
     use ratatui::backend::TestBackend;
     use ratatui::buffer::Buffer;
 
+    struct AppBuilder {
+        filename: String,
+        delimiter: Delimiter,
+        original_filename: Option<String>,
+        show_stats: bool,
+        echo_column: Option<String>,
+        ignore_case: bool,
+        no_headers: bool,
+        columns_regex: Option<String>,
+        filter_regex: Option<String>,
+        find_regex: Option<String>,
+    }
+
+    impl AppBuilder {
+        fn new(filename: &str) -> Self {
+            AppBuilder {
+                filename: filename.to_owned(),
+                delimiter: Delimiter::Default,
+                original_filename: None,
+                show_stats: false,
+                echo_column: None,
+                ignore_case: false,
+                no_headers: false,
+                columns_regex: None,
+                filter_regex: None,
+                find_regex: None,
+            }
+        }
+
+        fn build(self) -> Result<App> {
+            App::new(
+                self.filename.as_str(),
+                self.delimiter,
+                self.original_filename,
+                self.show_stats,
+                self.echo_column,
+                self.ignore_case,
+                self.no_headers,
+                self.columns_regex,
+                self.filter_regex,
+                self.find_regex,
+            )
+        }
+
+        fn delimiter(mut self, delimiter: Delimiter) -> Self {
+            self.delimiter = delimiter;
+            self
+        }
+
+        fn ignore_case(mut self, ignore_case: bool) -> Self {
+            self.ignore_case = ignore_case;
+            self
+        }
+
+        fn no_headers(mut self, no_headers: bool) -> Self {
+            self.no_headers = no_headers;
+            self
+        }
+
+        fn columns_regex(mut self, columns: Option<String>) -> Self {
+            self.columns_regex = columns;
+            self
+        }
+
+        fn filter_regex(mut self, filter: Option<String>) -> Self {
+            self.filter_regex = filter;
+            self
+        }
+    }
+
     fn to_lines(buf: &Buffer) -> Vec<String> {
         let mut symbols: String = "".to_owned();
         let area = buf.area();
@@ -605,15 +796,7 @@ mod tests {
 
     #[test]
     fn test_simple() {
-        let mut app = App::new(
-            "tests/data/simple.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/simple.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(30, 10);
@@ -646,15 +829,7 @@ mod tests {
 
     #[test]
     fn test_scroll_horizontal() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(30, 10);
@@ -714,15 +889,7 @@ mod tests {
 
     #[test]
     fn test_filter_columns() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(80, 10);
@@ -752,15 +919,7 @@ mod tests {
 
     #[test]
     fn test_filter_columns_case_sensitive() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(80, 10);
@@ -790,15 +949,10 @@ mod tests {
 
     #[test]
     fn test_filter_columns_ignore_case() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            true,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv")
+            .ignore_case(true)
+            .build()
+            .unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(80, 10);
@@ -830,15 +984,9 @@ mod tests {
     fn test_extra_fields_in_some_rows() {
         // Test getting column widths should not fail on data with bad formatting (some rows having
         // more fields than the header)
-        let mut app = App::new(
-            "tests/data/bad_double_quote.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/bad_double_quote.csv")
+            .build()
+            .unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(35, 10);
@@ -864,15 +1012,10 @@ mod tests {
 
     #[test]
     fn test_sniff_delimiter() {
-        let mut app = App::new(
-            "tests/data/small.bsv",
-            Delimiter::Auto,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/small.bsv")
+            .delimiter(Delimiter::Auto)
+            .build()
+            .unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(30, 10);
@@ -898,15 +1041,9 @@ mod tests {
 
     #[test]
     fn test_multi_lines() {
-        let mut app = App::new(
-            "tests/data/multi_lines.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/multi_lines.csv")
+            .build()
+            .unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(50, 30);
@@ -989,15 +1126,7 @@ mod tests {
 
     #[test]
     fn test_column_widths_boundary_condition() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(120, 10);
@@ -1033,15 +1162,7 @@ mod tests {
 
     #[test]
     fn test_scroll_right_most() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(40, 10);
@@ -1069,15 +1190,7 @@ mod tests {
 
     #[test]
     fn test_scroll_left_most() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(40, 10);
@@ -1106,15 +1219,7 @@ mod tests {
 
     #[test]
     fn test_scroll_half_page() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(40, 10);
@@ -1164,15 +1269,7 @@ mod tests {
 
     #[test]
     fn test_resize_column() {
-        let mut app = App::new(
-            "tests/data/cities.csv",
-            Delimiter::Default,
-            None,
-            false,
-            None,
-            false,
-        )
-        .unwrap();
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
         thread::sleep(time::Duration::from_millis(100));
 
         let backend = TestBackend::new(80, 10);
@@ -1243,4 +1340,168 @@ mod tests {
         ];
         assert_eq!(lines, expected);
     }
+
+    #[test]
+    fn test_sorting() {
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
+        thread::sleep(time::Duration::from_millis(100));
+
+        let backend = TestBackend::new(80, 10);
+        let mut terminal = Terminal::new(backend).unwrap();
+
+        step_and_draw(&mut app, &mut terminal, Control::ToggleSelectionType);
+        step_and_draw(&mut app, &mut terminal, Control::ScrollRight);
+        step_and_draw(&mut app, &mut terminal, Control::ToggleSort);
+        thread::sleep(time::Duration::from_millis(200));
+        step_and_draw(&mut app, &mut terminal, Control::Nothing);
+
+        let actual_buffer = terminal.backend().buffer().clone();
+        let lines = to_lines(&actual_buffer);
+        let expected = vec![
+            "────────────────────────────────────────────────────────────────────────────────",
+            "        LatD    LatM [▾]      LatS    NS    LonD    LonM    LonS    EW    C…    ",
+            "─────┬──────────────────────────────────────────────────────────────────────────",
+            "118  │  44      1             12      N     92      27      35      W     R…    ",
+            "56   │  43      2             59      N     76      9       0       W     S…    ",
+            "49   │  39      2             59      N     95      40      11      W     T…    ",
+            "29   │  46      4             11      N     118     19      48      W     W…    ",
+            "83   │  32      4             48      N     81      5       23      W     S…    ",
+            "─────┴──────────────────────────────────────────────────────────────────────────",
+            "stdin [Row 118/128, Col 1/10]                                                   ",
+        ];
+        assert_eq!(lines, expected);
+    }
+
+    #[test]
+    fn test_sorting_with_filter() {
+        let mut app = AppBuilder::new("tests/data/cities.csv").build().unwrap();
+        thread::sleep(time::Duration::from_millis(100));
+
+        let backend = TestBackend::new(80, 10);
+        let mut terminal = Terminal::new(backend).unwrap();
+
+        step_and_draw(&mut app, &mut terminal, Control::ToggleSelectionType);
+        step_and_draw(&mut app, &mut terminal, Control::ScrollRight);
+        step_and_draw(&mut app, &mut terminal, Control::ToggleSort);
+
+        thread::sleep(time::Duration::from_millis(200));
+        step_and_draw(&mut app, &mut terminal, Control::Nothing);
+
+        step_and_draw(&mut app, &mut terminal, Control::Filter("San".into()));
+        step_and_draw(
+            &mut app,
+            &mut terminal,
+            Control::FilterColumns("Lat|City".into()),
+        );
+
+        thread::sleep(time::Duration::from_millis(200));
+        step_and_draw(&mut app, &mut terminal, Control::Nothing);
+
+        let actual_buffer = terminal.backend().buffer().clone();
+        let lines = to_lines(&actual_buffer);
+        let expected = vec![
+            "────────────────────────────────────────────────────────────────────────────────",
+            "       LatD    LatM [▾]      LatS    City                                       ",
+            "────┬──────────────────────────────────────────────────┬────────────────────────",
+            "94  │  34      6             36      San Bernardino    │                        ",
+            "90  │  37      20            24      San Jose          │                        ",
+            "95  │  29      25            12      San Antonio       │                        ",
+            "88  │  34      25            11      Santa Barbara     │                        ",
+            "86  │  38      26            23      Santa Rosa        │                        ",
+            "────┴──────────────────────────────────────────────────┴────────────────────────",
+            "stdin [Row 94/128, Col 1/4] [Filter \"San\": -/11] [Filter \"Lat|City\": 4/10 cols] ",
+        ];
+        assert_eq!(lines, expected);
+    }
+
+    #[test]
+    fn test_no_headers() {
+        let mut app = AppBuilder::new("tests/data/no_headers.csv")
+            .no_headers(true)
+            .build()
+            .unwrap();
+        thread::sleep(time::Duration::from_millis(100));
+
+        let backend = TestBackend::new(30, 10);
+        let mut terminal = Terminal::new(backend).unwrap();
+
+        step_and_draw(&mut app, &mut terminal, Control::Nothing);
+        for _ in 0..7 {
+            step_and_draw(&mut app, &mut terminal, Control::ScrollDown);
+        }
+
+        let expected = vec![
+            "──────────────────────────────",
+            "      1     2                 ",
+            "───┬──────────────┬───────────",
+            "4  │  A4    B4    │           ",
+            "5  │  A5    B5    │           ",
+            "6  │  A6    B6    │           ",
+            "7  │  A7    B7    │           ",
+            "8  │  A8    B8    │           ",
+            "───┴──────────────┴───────────",
+            "stdin [Row 8/20, Col 1/2]     ",
+        ];
+        let actual_buffer = terminal.backend().buffer().clone();
+        let lines = to_lines(&actual_buffer);
+        assert_eq!(lines, expected);
+    }
+
+    #[test]
+    fn test_cli_columns_option() {
+        let mut app = AppBuilder::new("tests/data/cities.csv")
+            .columns_regex(Some("Lat".to_string()))
+            .build()
+            .unwrap();
+        thread::sleep(time::Duration::from_millis(100));
+
+        let backend = TestBackend::new(80, 10);
+        let mut terminal = Terminal::new(backend).unwrap();
+        step_and_draw(&mut app, &mut terminal, Control::Nothing);
+
+        let expected = vec![
+            "────────────────────────────────────────────────────────────────────────────────",
+            "      LatD    LatM    LatS                                                      ",
+            "───┬──────────────────────────┬─────────────────────────────────────────────────",
+            "1  │  41      5       59      │                                                 ",
+            "2  │  42      52      48      │                                                 ",
+            "3  │  46      35      59      │                                                 ",
+            "4  │  42      16      12      │                                                 ",
+            "5  │  43      37      48      │                                                 ",
+            "───┴──────────────────────────┴─────────────────────────────────────────────────",
+            "stdin [Row 1/128, Col 1/3] [Filter \"Lat\": 3/10 cols]                            ",
+        ];
+        let actual_buffer = terminal.backend().buffer().clone();
+        let lines = to_lines(&actual_buffer);
+        assert_eq!(lines, expected);
+    }
+
+    #[test]
+    fn test_cli_filter_option() {
+        let mut app = AppBuilder::new("tests/data/cities.csv")
+            .filter_regex(Some("San".to_string()))
+            .build()
+            .unwrap();
+        thread::sleep(time::Duration::from_millis(100));
+
+        let backend = TestBackend::new(80, 10);
+        let mut terminal = Terminal::new(backend).unwrap();
+        step_and_draw(&mut app, &mut terminal, Control::Nothing);
+
+        let expected = vec![
+            "────────────────────────────────────────────────────────────────────────────────",
+            "       LatD    LatM    LatS    NS    LonD    LonM    LonS    EW    City         ",
+            "────┬───────────────────────────────────────────────────────────────────────────",
+            "86  │  38      26      23      N     122     43      12      W     Santa Ro…    ",
+            "87  │  35      40      48      N     105     56      59      W     Santa Fe     ",
+            "88  │  34      25      11      N     119     41      59      W     Santa Ba…    ",
+            "89  │  33      45      35      N     117     52      12      W     Santa Ana    ",
+            "90  │  37      20      24      N     121     52      47      W     San Jose     ",
+            "────┴───────────────────────────────────────────────────────────────────────────",
+            "stdin [Row 86/128, Col 1/10] [Filter \"San\": 1/11]                               ",
+        ];
+        let actual_buffer = terminal.backend().buffer().clone();
+        let lines = to_lines(&actual_buffer);
+        assert_eq!(lines, expected);
+    }
 }
