@@ -14,11 +14,18 @@ pub enum Delimiter {
 
 impl Delimiter {
     /// Create a Delimiter by parsing the command line argument for the delimiter
-    pub fn from_arg(delimiter_arg: &Option<String>) -> Result<Self> {
+    pub fn from_arg(delimiter_arg: &Option<String>, tab_separation: bool) -> Result<Self> {
+        if tab_separation {
+            return Ok(Delimiter::Character('\t'.try_into()?));
+        }
+
         if let Some(s) = delimiter_arg {
             if s == "auto" {
                 return Ok(Delimiter::Auto);
             }
+            if s == r"\t" {
+                return Ok(Delimiter::Character(b'\t'));
+            }
             let mut chars = s.chars();
             let c = chars.next().context("Delimiter should not be empty")?;
             if !c.is_ascii() {
@@ -28,7 +35,10 @@ impl Delimiter {
                 );
             }
             if chars.next().is_some() {
-                bail!("Delimiter should be exactly one character, got {}", s);
+                bail!(
+                    "Delimiter should be exactly one character (or \\t), got '{}'",
+                    s
+                );
             }
             Ok(Delimiter::Character(c.try_into()?))
         } else {
