@@ -6,8 +6,9 @@ mod find;
 mod help;
 mod history;
 mod input;
-mod ui;
 #[allow(dead_code)]
+mod sort;
+mod ui;
 mod util;
 mod view;
 mod wrap;
@@ -96,19 +97,35 @@ struct Args {
     filename: Option<String>,
 
     /// Delimiter character (comma by default) or "auto" to auto-detect the delimiter
-    #[clap(short, long)]
+    #[clap(short, long, value_name = "char")]
     delimiter: Option<String>,
 
     /// Use tab separation. Shortcut for -d '\t'.
     #[clap(short = 't', long)]
     tab_separated: bool,
 
+    /// Do not interpret the first row as headers.
+    #[clap(long)]
+    no_headers: bool,
+
+    /// Use this regex to select columns to display by default
+    #[arg(long, value_name = "regex")]
+    columns: Option<String>,
+
+    /// Use this regex to filter rows to display by default
+    #[arg(long, value_name = "regex")]
+    filter: Option<String>,
+
+    /// Use this regex to find and hightlight matches by default
+    #[arg(long, value_name = "regex")]
+    find: Option<String>,
+
     /// Searches ignore case. Ignored if any uppercase letters are present in the search string
     #[clap(short, long)]
     ignore_case: bool,
 
     /// Print the value of this column to stdout for the selected row
-    #[arg(long)]
+    #[arg(long, value_name = "column_name")]
     echo_column: Option<String>,
 
     /// Show stats for debugging
@@ -175,6 +192,10 @@ fn run_csvlens() -> Result<Option<String>> {
         show_stats,
         args.echo_column,
         args.ignore_case,
+        args.no_headers,
+        args.columns,
+        args.filter,
+        args.find,
     )?;
 
     let mut app_runner = AppRunner::new(app);
